@@ -1,5 +1,5 @@
 /*
-  Copyright 2007-2014 David Robillard <http://drobilla.net>
+  Copyright 2007-2019 David Robillard <http://drobilla.net>
 
   Permission to use, copy, modify, and/or distribute this software for any
   purpose with or without fee is hereby granted, provided that the above
@@ -21,14 +21,15 @@
 #ifndef LILV_LILV_H
 #define LILV_LILV_H
 
+#include "lv2/core/lv2.h"
+#include "lv2/urid/urid.h"
+
 #include <stdarg.h>
+#include <stdbool.h>
 #include <stddef.h>
 #include <stdint.h>
 #include <stdio.h>
 
-#include "lv2/lv2plug.in/ns/lv2core/lv2.h"
-#include "lv2/lv2plug.in/ns/ext/urid/urid.h"
-
 #ifdef LILV_SHARED
 #    ifdef _WIN32
 #        define LILV_LIB_IMPORT __declspec(dllimport)
@@ -45,7 +46,8 @@
 #else
 #    define LILV_API
 #endif
-#if __GNUC__ > 3 || (__GNUC__ == 3 && __GNUC_MINOR__ >= 1)
+#if defined(__GNUC__) && (__GNUC__ > 3 || \
+                          (__GNUC__ == 3 && __GNUC_MINOR__ >= 1))
 #    define LILV_DEPRECATED __attribute__((__deprecated__))
 #else
 #    define LILV_DEPRECATED
@@ -53,8 +55,6 @@
 
 #ifdef __cplusplus
 extern "C" {
-#else
-#   include <stdbool.h>
 #endif
 
 #define LILV_NS_DOAP "http://usefulinc.com/ns/doap#"
@@ -66,8 +66,10 @@ extern "C" {
 #define LILV_NS_RDFS "http://www.w3.org/2000/01/rdf-schema#"
 #define LILV_NS_XSD  "http://www.w3.org/2001/XMLSchema#"
 
+#define LILV_URI_ATOM_PORT    "http://lv2plug.in/ns/ext/atom#AtomPort"
 #define LILV_URI_AUDIO_PORT   "http://lv2plug.in/ns/lv2core#AudioPort"
 #define LILV_URI_CONTROL_PORT "http://lv2plug.in/ns/lv2core#ControlPort"
+#define LILV_URI_CV_PORT      "http://lv2plug.in/ns/lv2core#CVPort"
 #define LILV_URI_EVENT_PORT   "http://lv2plug.in/ns/ext/event#EventPort"
 #define LILV_URI_INPUT_PORT   "http://lv2plug.in/ns/lv2core#InputPort"
 #define LILV_URI_MIDI_EVENT   "http://lv2plug.in/ns/ext/midi#MidiEvent"
@@ -93,10 +95,11 @@ typedef void LilvNodes;          /**< set<Node>. */
 
 /**
    @defgroup lilv Lilv
-   Lilv is a simple yet powerful C API for using LV2 plugins.
 
-   For more information about LV2, see <http://lv2plug.in>.
-   For more information about Lilv, see <http://drobilla.net/software/lilv>.
+   A library for discovering and using LV2 plugins.
+
+   For more information about LV2, see <http://lv2plug.in/>.
+
    @{
 */
 
@@ -210,11 +213,11 @@ lilv_node_equals(const LilvNode* value, const LilvNode* other);
    <table>
    <caption>Example Turtle Tokens</caption>
    <tr><th>URI</th><td>&lt;http://example.org/foo &gt;</td></tr>
-   <tr><th>QName</td><td>doap:name</td></tr>
-   <tr><th>String</td><td>"this is a string"</td></tr>
-   <tr><th>Float</td><td>1.0</td></tr>
-   <tr><th>Integer</td><td>1</td></tr>
-   <tr><th>Boolean</td><td>true</td></tr>
+   <tr><th>QName</th><td>doap:name</td></tr>
+   <tr><th>String</th><td>"this is a string"</td></tr>
+   <tr><th>Float</th><td>1.0</td></tr>
+   <tr><th>Integer</th><td>1</td></tr>
+   <tr><th>Boolean</th><td>true</td></tr>
    </table>
 */
 LILV_API char*
@@ -350,7 +353,7 @@ lilv_node_as_bool(const LilvNode* value);
    @endcode
 */
 #define LILV_FOREACH(colltype, iter, collection) \
-	for (LilvIter* (iter) = lilv_ ## colltype ## _begin(collection); \
+	for (LilvIter* iter = lilv_ ## colltype ## _begin(collection); \
 	     !lilv_ ## colltype ## _is_end(collection, iter); \
 	     (iter) = lilv_ ## colltype ## _next(collection, iter))
 
@@ -461,7 +464,7 @@ lilv_nodes_get_first(const LilvNodes* collection);
    Return whether `values` contains `value`.
 */
 LILV_API bool
-lilv_nodes_contains(const LilvNodes* values, const LilvNode* value);
+lilv_nodes_contains(const LilvNodes* nodes, const LilvNode* value);
 
 /**
    Return a new LilvNodes that contains all nodes from both `a` and `b`.
@@ -528,12 +531,20 @@ lilv_world_new(void);
 */
 #define LILV_OPTION_DYN_MANIFEST "http://drobilla.net/ns/lilv#dyn-manifest"
 
+/**
+   Set application-specific LV2_PATH.  This overrides the LV2_PATH from the
+   environment, so that lilv will only look inside the given path.  This can be
+   used to make self-contained applications.
+*/
+#define LILV_OPTION_LV2_PATH "http://drobilla.net/ns/lilv#lv2-path"
+
 /**
    Set an option option for `world`.
 
    Currently recognized options:
    @ref LILV_OPTION_FILTER_LANG
    @ref LILV_OPTION_DYN_MANIFEST
+   @ref LILV_OPTION_LV2_PATH
 */
 LILV_API void
 lilv_world_set_option(LilvWorld*      world,
@@ -577,8 +588,8 @@ lilv_world_load_all(LilvWorld* world);
    other things) MUST be identified by URIs (not paths) in save files.
 */
 LILV_API void
-lilv_world_load_bundle(LilvWorld* world,
-                       LilvNode*  bundle_uri);
+lilv_world_load_bundle(LilvWorld*      world,
+                       const LilvNode* bundle_uri);
 
 /**
    Load all specifications from currently loaded bundles.
@@ -605,11 +616,11 @@ lilv_world_load_plugin_classes(LilvWorld* world);
 
    This unloads statements loaded by lilv_world_load_bundle().  Note that this
    is not necessarily all information loaded from the bundle.  If any resources
-   have been separately loaded with liv_world_load_resource(), they must be
+   have been separately loaded with lilv_world_load_resource(), they must be
    separately unloaded with lilv_world_unload_resource().
 */
 LILV_API int
-lilv_world_unload_bundle(LilvWorld* world, LilvNode* bundle_uri);
+lilv_world_unload_bundle(LilvWorld* world, const LilvNode* bundle_uri);
 
 /**
    Load all the data associated with the given `resource`.
@@ -705,6 +716,16 @@ lilv_world_ask(LilvWorld*      world,
                const LilvNode* predicate,
                const LilvNode* object);
 
+/**
+   Get an LV2 symbol for some subject.
+
+   This will return the lv2:symbol property of the subject if it is given
+   explicitly, and otherwise will attempt to derive a symbol from the URI.
+   @return A string node that is a valid LV2 symbol, or NULL on error.
+*/
+LILV_API LilvNode*
+lilv_world_get_symbol(LilvWorld* world, const LilvNode* subject);
+
 /**
    @}
    @name Plugin
@@ -750,7 +771,7 @@ lilv_plugin_get_uri(const LilvPlugin* plugin);
 
    Typical hosts should not need to use this function.
    Note this always returns a fully qualified URI.  If you want a local
-   filesystem path, use lilv_uri_to_path().
+   filesystem path, use lilv_file_uri_parse().
    @return a shared string which must not be modified or freed.
 */
 LILV_API const LilvNode*
@@ -760,7 +781,7 @@ lilv_plugin_get_bundle_uri(const LilvPlugin* plugin);
    Get the (resolvable) URIs of the RDF data files that define a plugin.
    Typical hosts should not need to use this function.
    Note this always returns fully qualified URIs.  If you want local
-   filesystem paths, use lilv_uri_to_path().
+   filesystem paths, use lilv_file_uri_parse().
    @return a list of complete URLs eg. "file:///foo/ABundle.lv2/aplug.ttl",
    which is shared and must not be modified or freed.
 */
@@ -770,7 +791,7 @@ lilv_plugin_get_data_uris(const LilvPlugin* plugin);
 /**
    Get the (resolvable) URI of the shared library for `plugin`.
    Note this always returns a fully qualified URI.  If you want a local
-   filesystem path, use lilv_uri_to_path().
+   filesystem path, use lilv_file_uri_parse().
    @return a shared string which must not be modified or freed.
 */
 LILV_API const LilvNode*
@@ -805,7 +826,7 @@ lilv_plugin_get_class(const LilvPlugin* plugin);
    Return value must be freed by caller with lilv_nodes_free().
 */
 LILV_API LilvNodes*
-lilv_plugin_get_value(const LilvPlugin* p,
+lilv_plugin_get_value(const LilvPlugin* plugin,
                       const LilvNode*   predicate);
 
 /**
@@ -814,8 +835,8 @@ lilv_plugin_get_value(const LilvPlugin* p,
    of the plugin.
 */
 LILV_API bool
-lilv_plugin_has_feature(const LilvPlugin* p,
-                        const LilvNode*   feature_uri);
+lilv_plugin_has_feature(const LilvPlugin* plugin,
+                        const LilvNode*   feature);
 
 /**
    Get the LV2 Features supported (required or optionally) by a plugin.
@@ -828,7 +849,7 @@ lilv_plugin_has_feature(const LilvPlugin* p,
    Returned value must be freed by caller with lilv_nodes_free().
 */
 LILV_API LilvNodes*
-lilv_plugin_get_supported_features(const LilvPlugin* p);
+lilv_plugin_get_supported_features(const LilvPlugin* plugin);
 
 /**
    Get the LV2 Features required by a plugin.
@@ -842,7 +863,7 @@ lilv_plugin_get_supported_features(const LilvPlugin* p);
    Return value must be freed by caller with lilv_nodes_free().
 */
 LILV_API LilvNodes*
-lilv_plugin_get_required_features(const LilvPlugin* p);
+lilv_plugin_get_required_features(const LilvPlugin* plugin);
 
 /**
    Get the LV2 Features optionally supported by a plugin.
@@ -853,13 +874,13 @@ lilv_plugin_get_required_features(const LilvPlugin* p);
    Return value must be freed by caller with lilv_nodes_free().
 */
 LILV_API LilvNodes*
-lilv_plugin_get_optional_features(const LilvPlugin* p);
+lilv_plugin_get_optional_features(const LilvPlugin* plugin);
 
 /**
    Return whether or not a plugin provides a specific extension data.
 */
 LILV_API bool
-lilv_plugin_has_extension_data(const LilvPlugin* p,
+lilv_plugin_has_extension_data(const LilvPlugin* plugin,
                                const LilvNode*   uri);
 
 /**
@@ -868,13 +889,13 @@ lilv_plugin_has_extension_data(const LilvPlugin* p,
    will return a value for without instantiating the plugin.
 */
 LILV_API LilvNodes*
-lilv_plugin_get_extension_data(const LilvPlugin* p);
+lilv_plugin_get_extension_data(const LilvPlugin* plugin);
 
 /**
    Get the number of ports on this plugin.
 */
 LILV_API uint32_t
-lilv_plugin_get_num_ports(const LilvPlugin* p);
+lilv_plugin_get_num_ports(const LilvPlugin* plugin);
 
 /**
    Get the port ranges (minimum, maximum and default values) for all ports.
@@ -891,7 +912,7 @@ lilv_plugin_get_num_ports(const LilvPlugin* p);
    repeated calls to lilv_port_get_range().
 */
 LILV_API void
-lilv_plugin_get_port_ranges_float(const LilvPlugin* p,
+lilv_plugin_get_port_ranges_float(const LilvPlugin* plugin,
                                   float*            min_values,
                                   float*            max_values,
                                   float*            def_values);
@@ -903,20 +924,18 @@ lilv_plugin_get_port_ranges_float(const LilvPlugin* p,
    OF THIS FUNCTION WITH NULL OR VERY NASTY THINGS WILL HAPPEN.
 */
 LILV_API uint32_t
-lilv_plugin_get_num_ports_of_class(const LilvPlugin* p,
+lilv_plugin_get_num_ports_of_class(const LilvPlugin* plugin,
                                    const LilvNode*   class_1, ...);
 
-#ifndef SWIG
 /**
    Variant of lilv_plugin_get_num_ports_of_class() that takes a va_list.
 
    This function calls va_arg() on `args` but does not call va_end().
 */
 LILV_API uint32_t
-lilv_plugin_get_num_ports_of_class_va(const LilvPlugin* p,
+lilv_plugin_get_num_ports_of_class_va(const LilvPlugin* plugin,
                                       const LilvNode*   class_1,
                                       va_list           args);
-#endif
 
 /**
    Return whether or not the plugin introduces (and reports) latency.
@@ -924,7 +943,7 @@ lilv_plugin_get_num_ports_of_class_va(const LilvPlugin* p,
    lilv_plugin_get_latency_port() ONLY if this function returns true.
 */
 LILV_API bool
-lilv_plugin_has_latency(const LilvPlugin* p);
+lilv_plugin_has_latency(const LilvPlugin* plugin);
 
 /**
    Return the index of the plugin's latency port.
@@ -936,7 +955,7 @@ lilv_plugin_has_latency(const LilvPlugin* p);
    rate output port that reports the latency for each cycle in frames.
 */
 LILV_API uint32_t
-lilv_plugin_get_latency_port_index(const LilvPlugin* p);
+lilv_plugin_get_latency_port_index(const LilvPlugin* plugin);
 
 /**
    Get a port on `plugin` by `index`.
@@ -1100,9 +1119,9 @@ lilv_port_get_properties(const LilvPlugin* plugin,
    Return whether a port has a certain property.
 */
 LILV_API bool
-lilv_port_has_property(const LilvPlugin* p,
+lilv_port_has_property(const LilvPlugin* plugin,
                        const LilvPort*   port,
-                       const LilvNode*   property_uri);
+                       const LilvNode*   property);
 
 /**
    Return whether a port supports a certain event type.
@@ -1111,7 +1130,7 @@ lilv_port_has_property(const LilvPlugin* p,
    ev:supportsEvent property with `event_type` as the value.
 */
 LILV_API bool
-lilv_port_supports_event(const LilvPlugin* p,
+lilv_port_supports_event(const LilvPlugin* plugin,
                          const LilvPort*   port,
                          const LilvNode*   event_type);
 
@@ -1178,7 +1197,7 @@ lilv_port_is_a(const LilvPlugin* plugin,
 LILV_API void
 lilv_port_get_range(const LilvPlugin* plugin,
                     const LilvPort*   port,
-                    LilvNode**        deflt,
+                    LilvNode**        def,
                     LilvNode**        min,
                     LilvNode**        max);
 
@@ -1205,13 +1224,13 @@ lilv_port_get_scale_points(const LilvPlugin* plugin,
    the plugin URI as the `subject` parameter.
    @param world The world.
    @param map URID mapper.
-   @param subject The subject of the state description (e.g. a preset URI).
+   @param node The subject of the state description (e.g. a preset URI).
    @return A new LilvState which must be freed with lilv_state_free(), or NULL.
 */
 LILV_API LilvState*
 lilv_state_new_from_world(LilvWorld*      world,
                           LV2_URID_Map*   map,
-                          const LilvNode* subject);
+                          const LilvNode* node);
 
 /**
    Load a state snapshot from a file.
@@ -1266,20 +1285,31 @@ typedef const void* (*LilvGetPortValueFunc)(const char* port_symbol,
 
    @param map The map to use for mapping URIs in state.
 
-   @param file_dir Directory of files created by the plugin earlier (or NULL).
-   This is for hosts that support file creation at any time with state
+   @param scratch_dir Directory of files created by the plugin earlier, or
+   NULL.  This is for hosts that support file creation at any time with state
    state:makePath.  These files will be copied as necessary to `copy_dir` and
    not be referred to directly in state (a temporary directory is appropriate).
 
-   @param copy_dir Directory of copies of files in `file_dir` (or NULL).  This
-   directory will have the same structure as `file_dir` but with possibly
-   modified file names to distinguish different revisions.  If you only care
-   about saving one state snapshot, it can be the same as `save_dir`.  Plugin
-   state will refer to files in this directory.
+   @param copy_dir Directory of copies of files in `scratch_dir`, or NULL.
+   This directory will have the same structure as `scratch_dir` but with
+   possibly modified file names to distinguish revisions.  This allows the
+   saved state to contain the exact contents of the scratch file at save time,
+   so that the state is not ruined if the file is later modified (for example,
+   by the plugin continuing to record).  This can be the same as `save_dir` to
+   create a copy in the state bundle, but can also be a separate directory
+   which allows multiple state snapshots to share a single copy if the file has
+   not changed.
+
+   @param link_dir Directory of links to external files, or NULL.  A link will
+   be made in this directory to any external files referred to in plugin state.
+   In turn, links will be created in the save directory to these links (e.g.
+   save_dir/file => link_dir/file => /foo/bar/file).  This allows many state
+   snapshots to share a single link to an external file, so archival (e.g. with
+   tar -h) will not create several copies of the file.  If this is not
+   required, it can be the same as `save_dir`.
 
    @param save_dir Directory of files created by plugin during save (or NULL).
-   If the state will be saved, this should be the bundle directory later passed
-   to lilv_state_save().
+   This is typically the bundle directory later passed to lilv_state_save().
 
    @param get_value Function to get port values (or NULL).  If NULL, the
    returned state will not represent port values.  This should only be NULL in
@@ -1287,14 +1317,6 @@ typedef const void* (*LilvGetPortValueFunc)(const char* port_symbol,
 
    @param user_data User data to pass to `get_value`.
 
-   @param link_dir Directory of links to external files (or NULL).  A link will
-   be made in this directory to any external files referred to in plugin state.
-   In turn, links will be created in the save directory to these links (e.g.
-   save_dir/file => link_dir/file => /foo/bar/file).  This allows many state
-   snapshots to share a single link to an external file, so archival
-   (e.g. with tar -h) will not create several copies of the file.  If this is
-   not required, it can be the same as save_dir.
-
    @param flags Bitwise OR of LV2_State_Flags values.
 
    @param features Features to pass LV2_State_Interface.save().
@@ -1312,12 +1334,12 @@ typedef const void* (*LilvGetPortValueFunc)(const char* port_symbol,
    saving an instances state many times while avoiding any duplication of data.
 
    If supported (via state:makePath passed to LV2_Descriptor::instantiate()),
-   `file_dir` should be the directory where any files created by the plugin
+   `scratch_dir` should be the directory where any files created by the plugin
    (not during save time, e.g. during instantiation) are stored.  These files
-   will be copied to preserve their state at this time.plugin-created files are stored.
-   Lilv will assume any files within this directory (recursively) are created
-   by the plugin and all other files are immutable.  Note that this function
-   does not save the state, use lilv_state_save() for that.
+   will be copied to preserve their state at this time.plugin-created files are
+   stored.  Lilv will assume any files within this directory (recursively) are
+   created by the plugin and all other files are immutable.  Note that this
+   function does not save the state, use lilv_state_save() for that.
 
    See <a href="http://lv2plug.in/ns/ext/state/state.h">state.h</a> from the
    LV2 State extension for details on the `flags` and `features` parameters.
@@ -1326,7 +1348,7 @@ LILV_API LilvState*
 lilv_state_new_from_instance(const LilvPlugin*          plugin,
                              LilvInstance*              instance,
                              LV2_URID_Map*              map,
-                             const char*                file_dir,
+                             const char*                scratch_dir,
                              const char*                copy_dir,
                              const char*                link_dir,
                              const char*                save_dir,
@@ -1380,6 +1402,28 @@ LILV_API void
 lilv_state_set_label(LilvState*  state,
                      const char* label);
 
+/**
+   Set a metadata property on `state`.
+   @param state The state to set the metadata for.
+   @param key The key to store `value` under (URID).
+   @param value Pointer to the value to be stored.
+   @param size The size of `value` in bytes.
+   @param type The type of `value` (URID).
+   @param flags LV2_State_Flags for `value`.
+   @return 0 on success.
+
+   This is a generic version of lilv_state_set_label(), which sets metadata
+   properties visible to hosts, but not plugins.  This allows storing useful
+   information such as comments or preset banks.
+*/
+LILV_API int
+lilv_state_set_metadata(LilvState*  state,
+                        uint32_t    key,
+                        const void* value,
+                        size_t      size,
+                        uint32_t    type,
+                        uint32_t    flags);
+
 /**
    Function to set a port value.
    @param port_symbol The symbol of the port.
@@ -1644,8 +1688,9 @@ lilv_instance_connect_port(LilvInstance* instance,
 static inline void
 lilv_instance_activate(LilvInstance* instance)
 {
-	if (instance->lv2_descriptor->activate)
+	if (instance->lv2_descriptor->activate) {
 		instance->lv2_descriptor->activate(instance->lv2_handle);
+	}
 }
 
 /**
@@ -1668,8 +1713,9 @@ lilv_instance_run(LilvInstance* instance,
 static inline void
 lilv_instance_deactivate(LilvInstance* instance)
 {
-	if (instance->lv2_descriptor->deactivate)
+	if (instance->lv2_descriptor->deactivate) {
 		instance->lv2_descriptor->deactivate(instance->lv2_handle);
+	}
 }
 
 /**
@@ -1681,10 +1727,11 @@ static inline const void*
 lilv_instance_get_extension_data(const LilvInstance* instance,
                                  const char*         uri)
 {
-	if (instance->lv2_descriptor->extension_data)
+	if (instance->lv2_descriptor->extension_data) {
 		return instance->lv2_descriptor->extension_data(uri);
-	else
+	} else {
 		return NULL;
+	}
 }
 
 /**
